require "import"
import "android.widget.*"
import "android.view.View"
import "android.media.MediaPlayer"
import "android.net.Uri"
import "android.content.Context"
import "android.os.Environment"
import "android.text.InputType"
import "java.io.File"
import "java.io.FileOutputStream"
import "android.content.Intent"
import "android.os.Handler"
import "android.view.inputmethod.InputMethodManager"
import "android.app.NotificationManager"
import "android.app.Notification"
import "android.app.NotificationChannel"
import "android.os.Build"
import "android.net.ConnectivityManager"
import "android.net.NetworkInfo"
import "android.app.PendingIntent"
import "android.content.BroadcastReceiver"
import "android.content.SharedPreferences"
import "android.view.KeyEvent"
import "android.content.DialogInterface"
import "android.content.IntentFilter"
import "android.speech.tts.TextToSpeech"
import "android.speech.tts.UtteranceProgressListener"
import "android.speech.tts.Voice"
import "java.util.Locale"
import "java.util.HashMap"
import "java.util.Set"
import "cjson"

local handler = Handler()
local VIBRATION_DURATION = 100
local generationInProgress = false
local shouldShowIndependentInterface = false
local shouldCancelGeneration = false
local currentTempFiles = {}
local savedFileName = ""
local savedAudioData = nil
local savedAudioPath = nil
local currentErrorMessage = ""
local notificationManager = service.getSystemService(Context.NOTIFICATION_SERVICE)
local CHANNEL_ID = "tts_generation"
local NOTIFICATION_ID = 1
local cancelReceiver = nil
local MAX_RETRIES = 3
local RETRY_DELAY = 2000
local vibrationEnabled = true
local multiVoiceEnabled = false
local sharedPrefs = service.getSharedPreferences("tts_preferences", Context.MODE_PRIVATE)
local systemTTS = nil
local systemVoices = {}
local systemVoicesLoaded = false
local ttsInitialized = false

local elevenLabsVoices = {
{name = "Adam", code = "pNInz6obpgDQGcFmaJgB", id = "ADM"},
{name = "Aria", code = "21m00Tcm4TlvDq8ikWAM", id = "ARI"},
{name = "Roger", code = "CwhRBWXzGAHq8TQ4Fs17", id = "ROG"},
{name = "Sarah", code = "EXAVITQu4vr4xnSDxMaL", id = "SAR"},
{name = "Laura", code = "Yko7PKHZNXotIFUBG7I9", id = "LAU"},
{name = "Charlie", code = "IKne3meq5aSn9XLyUdCD", id = "CHA"},
{name = "George", code = "JBFqnCBsd6RMkjVDRZzb", id = "GEO"},
{name = "Callum", code = "N2lVS1w4EtoT3dr4eOWO", id = "CAL"},
{name = "River", code = "LcfcDJNUP1GQjkzn1xUU", id = "RIV"},
{name = "Liam", code = "TX3LPaxmHKxFdv7VOQHJ", id = "LIA"},
{name = "Charlotte", code = "XB0fDUnXU5powFXDhCwa", id = "CHR"},
{name = "Alice", code = "ErXwobaYiN019PkySvjV", id = "ALI"},
{name = "Matilda", code = "XrExE9yKIg1WjnnlVkGX", id = "MAT"},
{name = "Will", code = "bVMeCyTHy58xNoL34h3p", id = "WIL"},
{name = "Jessica", code = "KZyXbCR19Q4tzr1zYhNH", id = "JES"},
{name = "Eric", code = "VR6AewLTigWG4xSOukaG", id = "ERI"},
{name = "Chris", code = "iP95p4xoKVk53GoZ742B", id = "CHI"},
{name = "Brian", code = "nPczCjzI2devNBz1zQrb", id = "BRI"},
{name = "Daniel", code = "onwK4e9ZLuTAKqWW03F9", id = "DAN"},
{name = "Lily", code = "pFZP5JQG7iQjIQuC4Bku", id = "LIL"},
{name = "Bill", code = "zrHiDhphv9ZnVXBqCLjz", id = "BIL"}
}

function loadVibrationPreference()
vibrationEnabled = sharedPrefs.getBoolean("vibration_enabled", true)
end

function saveVibrationPreference(enabled)
local editor = sharedPrefs.edit()
editor.putBoolean("vibration_enabled", enabled)
editor.apply()
vibrationEnabled = enabled
end

function loadMultiVoicePreference()
multiVoiceEnabled = sharedPrefs.getBoolean("multi_voice_enabled", false)
end

function saveMultiVoicePreference(enabled)
local editor = sharedPrefs.edit()
editor.putBoolean("multi_voice_enabled", enabled)
editor.apply()
multiVoiceEnabled = enabled
end

function vibrate()
if not vibrationEnabled then return end
pcall(function()
local vibrator = service.getSystemService(Context.VIBRATOR_SERVICE)
if vibrator then vibrator.vibrate(VIBRATION_DURATION) end
end)
end

function updateExitButtonText()
exitButton.text = generationInProgress and "Salir y generar audio en segundo plano" or "Salir"
end

function updateDialogTitle()
local selectedService = serviceSpinner.getSelectedItem()
if selectedService == "TikTok" then
dlg.setTitle("Generador TTS de TikTok")
elseif selectedService == "ElevenLabs" then
dlg.setTitle("Generador TTS de ElevenLabs")
else
dlg.setTitle("Generador TTS del Sistema")
end
end

function updateMultiVoiceButtonText()
multiVoiceButton.text = multiVoiceEnabled and "Múltiples Voces: ON" or "Múltiples Voces: OFF"
end

function updateVoiceRowVisibility()
voiceScrollView.visibility = multiVoiceEnabled and View.VISIBLE or View.GONE
end

function insertVoiceTag(voiceName)
local currentText = textInput.text or ""
local cursorPos = textInput.getSelectionStart()
local tagText = "{{@" .. voiceName .. "}}\n"
local newText = string.sub(currentText, 1, cursorPos) .. tagText .. string.sub(currentText, cursorPos + 1)
textInput.setText(newText)
textInput.setSelection(cursorPos + #tagText)
end

function parseTextWithVoices(text)
if not multiVoiceEnabled then return {{text = text, voice = nil}} end
local segments = {}
local currentPos = 1
local defaultVoice = nil
while currentPos <= #text do
local tagStart, tagEnd = string.find(text, "{{@(%w+)}}", currentPos)
if not tagStart then
if currentPos <= #text then
table.insert(segments, {text = string.sub(text, currentPos), voice = defaultVoice})
end
break
end
if tagStart > currentPos then
table.insert(segments, {text = string.sub(text, currentPos, tagStart - 1), voice = defaultVoice})
end
local voiceName = string.match(text, "{{@(%w+)}}", tagStart)
for _, voice in ipairs(elevenLabsVoices) do
if voice.name == voiceName then
defaultVoice = voice.code
break
end
end
currentPos = tagEnd + 1
end
return segments
end

function initializeSystemTTS()
pcall(function()
if systemTTS then systemTTS.shutdown() end
ttsInitialized = false
systemTTS = TextToSpeech(service, TextToSpeech.OnInitListener {
onInit = function(status)
if status == TextToSpeech.SUCCESS then
ttsInitialized = true
systemTTS.setSpeechRate(1.2)
systemTTS.setPitch(1.0)
loadSystemVoices()
else
systemVoicesLoaded = true
ttsInitialized = false
end
end
})
end)
end

function loadSystemVoices()
pcall(function()
systemVoices = {}
if systemTTS and ttsInitialized then
if Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP then
local voices = systemTTS.getVoices()
if voices then
local voiceIterator = voices.iterator()
while voiceIterator.hasNext() do
local voice = voiceIterator.next()
local voiceName = voice.getName()
local locale = voice.getLocale()
local displayName = voiceName .. " (" .. locale.getDisplayLanguage() .. ")"
table.insert(systemVoices, {name = displayName, voiceId = voiceName, locale = locale, voice = voice})
end
end
end
if #systemVoices == 0 then
local availableLanguages = systemTTS.getAvailableLanguages()
if availableLanguages then
local langIterator = availableLanguages.iterator()
local voiceCount = 0
while langIterator.hasNext() and voiceCount < 10 do
local locale = langIterator.next()
local displayName = "TTS Sistema - " .. locale.getDisplayLanguage()
table.insert(systemVoices, {name = displayName, voiceId = "system_voice_" .. voiceCount, locale = locale, voice = nil})
voiceCount = voiceCount + 1
end
end
end
end
systemVoicesLoaded = true
end)
end

function generateSystemTTSAudio(text, voiceInfo, callback, retryCount)
retryCount = retryCount or 0
pcall(function()
if not systemTTS or not ttsInitialized then
if retryCount < 2 then
handler.postDelayed(Runnable {
run = function()
initializeSystemTTS()
handler.postDelayed(Runnable {
run = function()
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
end
}, 1500)
end
}, 500)
return
else
callback(nil, "Sistema TTS no se pudo inicializar después de varios intentos")
return
end
end
if not text or #text == 0 then
callback(nil, "Texto vacío para TTS")
return
end
if #text > 500 then text = string.sub(text, 1, 500) end
local tempFilePath = service.getCacheDir().getPath() .. "/system_tts_temp_" .. os.time() .. "_" .. math.random(1000, 9999) .. ".wav"
local tempFile = File(tempFilePath)
if tempFile.exists() then tempFile.delete() end
local utteranceId = "tts_gen_" .. os.time() .. "_" .. math.random(10000, 99999)
local params = HashMap()
params.put(TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID, utteranceId)
params.put(TextToSpeech.Engine.KEY_PARAM_VOLUME, "1.0")
local voiceSet = false
if Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP and voiceInfo.voice then
local result = systemTTS.setVoice(voiceInfo.voice)
if result == TextToSpeech.SUCCESS then voiceSet = true end
end
if not voiceSet and voiceInfo.locale then
local result = systemTTS.setLanguage(voiceInfo.locale)
if result == TextToSpeech.LANG_MISSING_DATA or result == TextToSpeech.LANG_NOT_SUPPORTED then
callback(nil, "Idioma no soportado: " .. voiceInfo.locale.getDisplayLanguage())
return
end
end
local audioGenerated = false
local errorOccurred = false
local timeoutHandler = nil
local progressChecker = nil
systemTTS.setOnUtteranceProgressListener(UtteranceProgressListener {
onStart = function(currentUtteranceId)
if currentUtteranceId == utteranceId then
if timeoutHandler then
handler.removeCallbacks(timeoutHandler)
timeoutHandler = nil
end
if progressChecker then handler.removeCallbacks(progressChecker) end
progressChecker = Runnable {
run = function()
local file = File(tempFilePath)
if file.exists() and file.length() > 0 and not audioGenerated and not errorOccurred then
audioGenerated = true
local fileStream = io.open(tempFilePath, "rb")
if fileStream then
local content = fileStream:read("*all")
fileStream:close()
file.delete()
if content and #content > 50 then
callback(content, nil)
else
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "Audio generado muy pequeño")
end
end
else
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "No se pudo leer el archivo de audio")
end
end
end
end
}
handler.postDelayed(progressChecker, 2000)
end
end,
onDone = function(currentUtteranceId)
if currentUtteranceId == utteranceId and not audioGenerated and not errorOccurred then
audioGenerated = true
if timeoutHandler then
handler.removeCallbacks(timeoutHandler)
timeoutHandler = nil
end
if progressChecker then
handler.removeCallbacks(progressChecker)
progressChecker = nil
end
handler.postDelayed(Runnable {
run = function()
local file = File(tempFilePath)
if file.exists() and file.length() > 0 then
local fileStream = io.open(tempFilePath, "rb")
if fileStream then
local content = fileStream:read("*all")
fileStream:close()
file.delete()
if content and #content > 50 then
callback(content, nil)
else
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "Audio generado muy pequeño o vacío")
end
end
else
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "No se pudo leer el archivo de audio generado")
end
end
else
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "No se generó el archivo de audio correctamente")
end
end
end
}, 800)
end
end,
onError = function(currentUtteranceId, errorCode)
if currentUtteranceId == utteranceId and not audioGenerated and not errorOccurred then
errorOccurred = true
if timeoutHandler then
handler.removeCallbacks(timeoutHandler)
timeoutHandler = nil
end
if progressChecker then
handler.removeCallbacks(progressChecker)
progressChecker = nil
end
if retryCount < 2 then
handler.postDelayed(Runnable {
run = function()
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
end
}, 1500)
else
local errorMsg = "Error TTS código: " .. (errorCode or "desconocido") .. " después de " .. (retryCount + 1) .. " intentos"
callback(nil, errorMsg)
end
end
end
})
timeoutHandler = Runnable {
run = function()
if not audioGenerated and not errorOccurred then
errorOccurred = true
if progressChecker then
handler.removeCallbacks(progressChecker)
progressChecker = nil
end
if retryCount < 2 then
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
else
callback(nil, "Tiempo de espera agotado después de " .. (retryCount + 1) .. " intentos - Intente con un texto más corto")
end
end
end
}
handler.postDelayed(timeoutHandler, 45000)
local result = systemTTS.synthesizeToFile(text, params, tempFilePath)
if result ~= TextToSpeech.SUCCESS then
if timeoutHandler then handler.removeCallbacks(timeoutHandler) end
if progressChecker then handler.removeCallbacks(progressChecker) end
if retryCount < 2 then
handler.postDelayed(Runnable {
run = function()
generateSystemTTSAudio(text, voiceInfo, callback, retryCount + 1)
end
}, 800)
else
callback(nil, "Error al iniciar síntesis TTS después de " .. (retryCount + 1) .. " intentos - código: " .. result)
end
return
end
end)
end

function isNetworkAvailable()
local result = false
pcall(function()
local cm = service.getSystemService(Context.CONNECTIVITY_SERVICE)
if cm then
local activeNetwork = cm.getActiveNetworkInfo()
result = activeNetwork and activeNetwork.isConnectedOrConnecting()
end
end)
return result
end

function cancelGeneration()
shouldCancelGeneration = true
generationInProgress = false
cleanupTempFiles(currentTempFiles)
currentTempFiles = {}
hideProgressNotification()
generateAudioButton.text = "Generar Audio"
updateResultText("Generación cancelada por el usuario")
toggleAudioControls(false)
updateExitButtonText()
pcall(function()
if cancelReceiver then
service.unregisterReceiver(cancelReceiver)
cancelReceiver = nil
end
end)
end

function setupCancelReceiver()
pcall(function()
if cancelReceiver then
service.unregisterReceiver(cancelReceiver)
cancelReceiver = nil
end
cancelReceiver = BroadcastReceiver {
onReceive = function(context, intent)
if intent.getAction() == "CANCEL_TTS_GENERATION" then
cancelGeneration()
end
end
}
local filter = IntentFilter("CANCEL_TTS_GENERATION")
service.registerReceiver(cancelReceiver, filter)
end)
end

function sanitizeFileName(fileName)
if not fileName or #fileName == 0 then return "audio_generado" end
fileName = string.gsub(fileName, "[\\/:*?\"<>|]", "_")
fileName = string.gsub(fileName, "^%s+", "")
fileName = string.gsub(fileName, "%s+$", "")
if #fileName == 0 then return "audio_generado" end
return fileName
end

function createNotificationChannel()
pcall(function()
if Build.VERSION.SDK_INT >= Build.VERSION_CODES.O then
local channel = NotificationChannel(CHANNEL_ID, "Generación TTS", NotificationManager.IMPORTANCE_LOW)
channel.setDescription("Progreso de generación de audio")
notificationManager.createNotificationChannel(channel)
end
end)
end

function showProgressNotification(title, message, isError)
pcall(function()
createNotificationChannel()
local builder = Notification.Builder(service, CHANNEL_ID)
builder.setContentTitle(title)
if isError then
builder.setContentText(message)
builder.setSmallIcon(android.R.drawable.ic_dialog_alert)
builder.setOngoing(false)
else
builder.setContentText(message .. " • Haz clic para cancelar")
builder.setSmallIcon(android.R.drawable.ic_media_play)
builder.setOngoing(true)
local cancelIntent = Intent("CANCEL_TTS_GENERATION")
cancelIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK)
local flags = PendingIntent.FLAG_UPDATE_CURRENT
if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
flags = flags | PendingIntent.FLAG_IMMUTABLE
end
local pendingIntent = PendingIntent.getBroadcast(service, 0, cancelIntent, flags)
builder.setContentIntent(pendingIntent)
end
notificationManager.notify(NOTIFICATION_ID, builder.build())
end)
end

function hideProgressNotification()
pcall(function()
notificationManager.cancel(NOTIFICATION_ID)
end)
end

function removeUrls(text)
if not text then return "" end
text = string.gsub(text, "https?://[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]+", "")
text = string.gsub(text, "www%.[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]+", "")
text = string.gsub(text, "[%w%.%-_]+%.[%w%.%-_]+/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "t%.me/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "chat%.whatsapp%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "wa%.me/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "youtube%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "youtu%.be/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "instagram%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "facebook%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "twitter%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "x%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "tiktok%.com/[%w%-%._~:/?#%[%]@!$&'%(%)%*%+,;=%%]*", "")
text = string.gsub(text, "@[%w%.%-_]+", "")
text = string.gsub(text, "#[%w%.%-_]+", "")
return text
end

function sanitizeText(text)
if not text then return "" end
text = removeUrls(text)
text = string.gsub(text, "\r\n", " ")
text = string.gsub(text, "\n", " ")
text = string.gsub(text, "\r", " ")
text = string.gsub(text, "[^a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ¿¡.,;:()!?%s-{}@]", "")
text = string.gsub(text, "%s+", " ")
text = string.gsub(text, "^%s+", "")
text = string.gsub(text, "%s+$", "")
if #text == 0 then return "Sin contenido válido para procesar" end
return text
end

function urlEncode(str)
if not str then return "" end
str = sanitizeText(str)
local function escape(s)
s = string.gsub(s, "([^%w %-%_%.%~])", function(c) return string.format("%%%02X", string.byte(c)) end)
return string.gsub(s, " ", "+")
end
return escape(str)
end

function splitText(text, maxLength)
local chunks = {}
local totalLength = #text
local position = 1
while position <= totalLength do
local endPos = math.min(position + maxLength - 1, totalLength)
if endPos < totalLength then
local lastSpace = string.find(string.reverse(string.sub(text, position, endPos)), "%s")
if lastSpace then endPos = endPos - lastSpace + 1 end
end
table.insert(chunks, string.sub(text, position, endPos))
position = endPos + 1
end
return chunks
end

function combineAudioFiles(files)
local outputPath = service.getCacheDir().getPath() .. "/combined_audio.mp3"
local outputFile = io.open(outputPath, "wb")
if not outputFile then return nil end
for _, filePath in ipairs(files) do
local inputFile = io.open(filePath, "rb")
if inputFile then
local content = inputFile:read("*all")
outputFile:write(content)
inputFile:close()
end
end
outputFile:close()
return outputPath
end

function cleanupTempFiles(tempFiles)
if tempFiles then
for _, filePath in ipairs(tempFiles) do
pcall(function()
local file = File(filePath)
if file.exists() then file.delete() end
end)
end
end
end

function saveAudioToExternalStorage(audioData, fileName)
local result = nil
pcall(function()
local baseFolderPath = Environment.getExternalStorageDirectory().getPath() .. "/Download/Generador TTS de Tiktok"
local folder = File(baseFolderPath)
if not folder.exists() then folder.mkdirs() end
local cleanFileName = sanitizeFileName(fileName)
local filePath = baseFolderPath .. "/" .. cleanFileName .. ".mp3"
local file = io.open(filePath, "wb")
if file then
file:write(audioData)
file:close()
result = filePath
end
end)
return result
end

function shareAudioFile(audioData, fileName)
if not audioData then
service.speak("No hay audio disponible para compartir")
return
end
local finalFileName = sanitizeFileName(fileName)
local filePath = saveAudioToExternalStorage(audioData, finalFileName)
if filePath then
pcall(function()
service.shareFile(filePath)
service.speak("Compartiendo audio")
end)
else
service.speak("Error al preparar el archivo para compartir")
end
end

function getErrorMessage(code)
local errorMessages = {
[400] = "Solicitud incorrecta - Verifique el texto y modelo seleccionado",
[401] = "No autorizado - Problema con la autenticación del servicio",
[403] = "Prohibido - Acceso denegado al servicio",
[404] = "No encontrado - El servicio o modelo no está disponible",
[429] = "Demasiadas solicitudes - Intente nuevamente en unos minutos",
[500] = "Error interno del servidor - El servicio TTS está experimentando problemas",
[502] = "Gateway incorrecto - Problema de conexión con el servicio",
[503] = "Servicio no disponible - El servidor está temporalmente fuera de servicio",
[504] = "Tiempo de espera agotado - El servidor tardó demasiado en responder"
}
return errorMessages[code] or ("Error " .. code .. " - Error desconocido del servidor")
end

function makeHttpRequestWithRetry(url, callback, retryCount)
retryCount = retryCount or 0
if shouldCancelGeneration then return end
Http.get(url, function(code, content)
if shouldCancelGeneration then return end
if code == 200 then
callback(code, content)
elseif (code == 500 or code == 502 or code == 503 or code == 504) and retryCount < MAX_RETRIES then
local nextRetry = retryCount + 1
handler.postDelayed(Runnable {
run = function()
if not shouldCancelGeneration then
makeHttpRequestWithRetry(url, callback, nextRetry)
end
end
}, RETRY_DELAY * nextRetry)
else
callback(code, content)
end
end)
end

function generateElevenLabsAudio(text, voice, style, callback)
if not text then
callback(nil, "Valores de parámetros inválidos")
return
end
text = text:gsub("\n", " "):gsub("\"", "\\\"")
local stability = 0.5
local similarity_boost = 0.5
if style == "Estable" then
stability = 0.8
similarity_boost = 0.3
elseif style == "Creativo" then
stability = 0.3
similarity_boost = 0.8
elseif style == "Equilibrado" then
stability = 0.5
similarity_boost = 0.5
end
local url = "https://ct-api.wondershare.cc/v2/ctsrv/relay_app"
local headers = {
["User-Agent"] = "com.wondershare.filmorago13861",
["Accept-Encoding"] = "gzip",
["Content-Type"] = "application/json",
["x-client-type"] = "4",
["x-client-sn"] = "90cd7ace8c50c6c36b5d68ca79c4b8a0",
["x-app-key"] = "a57b8e91c3a5ef000619cd9f910f4be7",
["x-prod-id"] = "1937",
["x-prod-ver"] = "13.8.61",
["x-plat-id"] = "1937",
["x-prod-name"] = "FilmoraV13_Android",
["x-ver"] = "",
["x-lang"] = "en-us",
["x-user-type"] = "1937",
["authorization"] = "Bearer 0poZPNlJlTF6u5DAt-qCI9w",
["x-signature"] = "7d851229f2ec0104b76c74e642aa3d70baa676cfe81c1d902e02b0cd4344991a",
["x-timestamp"] = "1734040459848"
}
local initial_payload = string.format([[{"lang_code":"en-US","model_id":"eleven_monolingual_v1","pitch_rate":0,"platform_id":10,"speech_rate":0,"task_contents":[{"sentence":"%s","task_index":1}],"url_alias":"v5_voice_tts_internal_transfer_no_consume","ver":2,"voice_code":"%s","volume":0}]], text, voice)
Thread(Runnable{run=function()
Http.post(url, initial_payload, headers, function(code, response)
if shouldCancelGeneration then return end
if code ~= 200 then
callback(nil, "Error " .. code)
return
end
local data = cjson.decode(response)
local task_id = data.data and data.data.task_id
if not task_id then
callback(nil, "Error al obtener el ID de la tarea")
return
end
local task_headers = {}
for k, v in pairs(headers) do task_headers[k] = v end
task_headers['x-signature'] = "75c55f15b42b3c05ac98948015ee4d239a254875d90ec769fe86f4e27bf80121"
task_headers['x-timestamp'] = "1734040463969"
local task_payload = string.format([[{"task_id":"%s","url_alias":"v5_voice_tts_internal_result_task_id_no_consume"}]], task_id)
local function checkStatus()
if shouldCancelGeneration then return end
Http.post(url, task_payload, task_headers, function(code, response)
if shouldCancelGeneration then return end
if code ~= 200 then
callback(nil, "Error " .. code)
return
end
local data = cjson.decode(response)
local status = data.data and data.data.status
if status == 1 then
local download_url = data.data.list[1] and data.data.list[1].download_url
if download_url and download_url ~= "" then
Http.get(download_url, function(code, content)
if shouldCancelGeneration then return end
if code == 200 then
callback(content, nil)
else
callback(nil, "Error al descargar el audio: "..code)
end
end)
else
callback(nil, "No hay URL de descarga disponible")
end
elseif status == 2 then
handler.postDelayed(Runnable {
run = function()
if not shouldCancelGeneration then checkStatus() end
end
}, 2000)
else
callback(nil, "Fallo en la generación")
end
end)
end
checkStatus()
end)
end}).start()
end

function generateTextToAudio(text, speaker, style, callback)
if not text or not speaker then
callback(nil, "Parámetros inválidos - Texto o modelo no especificado")
return
end
local selectedService = serviceSpinner.getSelectedItem()
if selectedService == "Voces del sistema" then
shouldCancelGeneration = false
currentTempFiles = {}
text = sanitizeText(text)
if #text < 3 then
callback(nil, "Texto demasiado corto después de la limpieza - Ingrese más contenido")
return
end
local maxChunkSize = 400
local voiceInfo = nil
for i, voice in ipairs(systemVoices) do
if voice.name == speaker then
voiceInfo = voice
break
end
end
if not voiceInfo then
callback(nil, "Voz del sistema no encontrada")
return
end
if #text <= maxChunkSize then
showProgressNotification("Generando Audio TTS", "Procesando con TTS del sistema", false)
generateSystemTTSAudio(text, voiceInfo, function(audioData, error)
if shouldCancelGeneration then
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
generationInProgress = false
updateExitButtonText()
if error then
local errorMsg = "Error TTS Sistema: " .. error
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
else
hideProgressNotification()
callback(audioData, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
end
end)
return
end
local textChunks = splitText(text, maxChunkSize)
local totalChunks = #textChunks
local processedChunks = 0
local tempFiles = {}
currentTempFiles = tempFiles
local failed = false
local function updateProgress(message)
handler.post(Runnable {
run = function()
if not shouldShowIndependentInterface then updateResultText(message) end
end
})
end
local function processSystemChunk(index)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if index > totalChunks then
generationInProgress = false
updateExitButtonText()
updateProgress("Combinación de audio exitosa con " .. #tempFiles .. " fragmentos del sistema")
showProgressNotification("Generando Audio TTS", "Combinando " .. #tempFiles .. " fragmentos del sistema", false)
local combinedPath = combineAudioFiles(tempFiles)
if combinedPath then
local combinedFile = io.open(combinedPath, "rb")
if combinedFile then
local combinedContent = combinedFile:read("*all")
combinedFile:close()
cleanupTempFiles(tempFiles)
hideProgressNotification()
callback(combinedContent, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
return
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al leer el archivo combinado del sistema"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al combinar los archivos de audio del sistema"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
return
end
local chunk = textChunks[index]
updateProgress("Procesando fragmento del sistema " .. index .. " de " .. totalChunks)
showProgressNotification("Generando Audio TTS", "Fragmento del sistema " .. index .. " de " .. totalChunks, false)
generateSystemTTSAudio(chunk, voiceInfo, function(content, error)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if error then
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error en fragmento del sistema " .. index .. ": " .. error
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
local tempFile = service.getCacheDir().getPath() .. "/tts_system_chunk_" .. index .. ".wav"
local file = io.open(tempFile, "wb")
if file then
file:write(content)
file:close()
table.insert(tempFiles, tempFile)
processedChunks = processedChunks + 1
updateProgress("Completado fragmento del sistema " .. processedChunks .. " de " .. totalChunks)
if not failed and not shouldCancelGeneration then
handler.postDelayed(Runnable {
run = function()
if not shouldCancelGeneration then processSystemChunk(index + 1) end
end
}, 2000)
end
else
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error al guardar fragmento del sistema " .. index
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
end)
end
processSystemChunk(1)
return
end
if multiVoiceEnabled then
local segments = parseTextWithVoices(text)
if #segments > 1 then
local totalSegments = #segments
local processedSegments = 0
local tempFiles = {}
currentTempFiles = tempFiles
local failed = false
shouldCancelGeneration = false
local function updateProgress(message)
handler.post(Runnable {
run = function()
if not shouldShowIndependentInterface then updateResultText(message) end
end
})
end
local function processSegment(index)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if index > totalSegments then
generationInProgress = false
updateExitButtonText()
updateProgress("Combinación de audio exitosa con " .. #tempFiles .. " segmentos de múltiples voces")
showProgressNotification("Generando Audio TTS", "Combinando " .. #tempFiles .. " segmentos", false)
local combinedPath = combineAudioFiles(tempFiles)
if combinedPath then
local combinedFile = io.open(combinedPath, "rb")
if combinedFile then
local combinedContent = combinedFile:read("*all")
combinedFile:close()
cleanupTempFiles(tempFiles)
hideProgressNotification()
callback(combinedContent, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
return
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al leer el archivo combinado"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al combinar los archivos de audio"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
return
end
local segment = segments[index]
if not segment.text or #segment.text == 0 then
processSegment(index + 1)
return
end
local segmentText = sanitizeText(segment.text)
if #segmentText < 3 then
processSegment(index + 1)
return
end
local voiceCode = segment.voice or speaker
updateProgress("Procesando segmento " .. index .. " de " .. totalSegments)
showProgressNotification("Generando Audio TTS", "Segmento " .. index .. " de " .. totalSegments, false)
generateElevenLabsAudio(segmentText, voiceCode, style, function(content, error)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if error then
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error en segmento " .. index .. ": " .. error
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
local tempFile = service.getCacheDir().getPath() .. "/tts_segment_" .. index .. ".mp3"
local file = io.open(tempFile, "wb")
if file then
file:write(content)
file:close()
table.insert(tempFiles, tempFile)
processedSegments = processedSegments + 1
updateProgress("Completado segmento " .. processedSegments .. " de " .. totalSegments)
if not failed and not shouldCancelGeneration then
handler.postDelayed(Runnable {
run = function()
if not shouldCancelGeneration then processSegment(index + 1) end
end
}, 800)
end
else
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error al guardar segmento " .. index
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
end)
end
processSegment(1)
return
end
end
if not isNetworkAvailable() then
callback(nil, "Sin conexión a internet - Verifique su conexión y vuelva a intentar")
return
end
shouldCancelGeneration = false
currentTempFiles = {}
text = sanitizeText(text)
if #text < 3 then
callback(nil, "Texto demasiado corto después de la limpieza - Ingrese más contenido")
return
end
local isElevenLabs = string.len(speaker) == 20
local maxChunkSize = isElevenLabs and 500 or 300
if #text <= maxChunkSize then
showProgressNotification("Generando Audio TTS", "Procesando fragmento único", false)
if isElevenLabs then
generateElevenLabsAudio(text, speaker, style, function(audioData, error)
if shouldCancelGeneration then
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
generationInProgress = false
updateExitButtonText()
if error then
local errorMsg = "Error Eleven Labs: " .. error
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
hideProgressNotification()
callback(audioData, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
end)
else
local url = "https://fastrestapis.fasturl.cloud/tts/tiktok?text=" .. urlEncode(text) .. "&speaker=" .. speaker
makeHttpRequestWithRetry(url, function(code, content)
if shouldCancelGeneration then
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
generationInProgress = false
updateExitButtonText()
if code ~= 200 then
local errorMsg = getErrorMessage(code)
showProgressNotification("Error en Generación TTS", "Error: " .. errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
hideProgressNotification()
callback(content, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
end)
end
return
end
local textChunks = splitText(text, maxChunkSize)
local totalChunks = #textChunks
local processedChunks = 0
local tempFiles = {}
currentTempFiles = tempFiles
local failed = false
local function updateProgress(message)
handler.post(Runnable {
run = function()
if not shouldShowIndependentInterface then updateResultText(message) end
end
})
end
local function processChunk(index)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if index > totalChunks then
generationInProgress = false
updateExitButtonText()
updateProgress("Combinación de audio exitosa con " .. #tempFiles .. " fragmentos")
showProgressNotification("Generando Audio TTS", "Combinando " .. #tempFiles .. " fragmentos", false)
local combinedPath = combineAudioFiles(tempFiles)
if combinedPath then
local combinedFile = io.open(combinedPath, "rb")
if combinedFile then
local combinedContent = combinedFile:read("*all")
combinedFile:close()
cleanupTempFiles(tempFiles)
hideProgressNotification()
callback(combinedContent, nil)
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showIndependentInterface()
end
return
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al leer el archivo combinado"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
else
cleanupTempFiles(tempFiles)
local errorMsg = "Error al combinar los archivos de audio"
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
return
end
local chunk = textChunks[index]
updateProgress("Procesando fragmento " .. index .. " de " .. totalChunks)
showProgressNotification("Generando Audio TTS", "Fragmento " .. index .. " de " .. totalChunks, false)
if isElevenLabs then
generateElevenLabsAudio(chunk, speaker, style, function(content, error)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if error then
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error en fragmento " .. index .. ": " .. error
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
local tempFile = service.getCacheDir().getPath() .. "/tts_chunk_" .. index .. ".mp3"
local file = io.open(tempFile, "wb")
if file then
file:write(content)
file:close()
table.insert(tempFiles, tempFile)
processedChunks = processedChunks + 1
updateProgress("Completado fragmento " .. processedChunks .. " de " .. totalChunks)
if not failed and not shouldCancelGeneration then
handler.postDelayed(Runnable {
run = function()
if not shouldCancelGeneration then processChunk(index + 1) end
end
}, 1000)
end
else
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error al guardar fragmento " .. index
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
end)
else
local url = "https://fastrestapis.fasturl.cloud/tts/tiktok?text=" .. urlEncode(chunk) .. "&speaker=" .. speaker
handler.postDelayed(Runnable {
run = function()
if shouldCancelGeneration then return end
makeHttpRequestWithRetry(url, function(code, content)
if shouldCancelGeneration then
cleanupTempFiles(tempFiles)
generationInProgress = false
hideProgressNotification()
updateExitButtonText()
return
end
if code ~= 200 then
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error en fragmento " .. index .. ": " .. getErrorMessage(code)
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
return
end
local tempFile = service.getCacheDir().getPath() .. "/tts_chunk_" .. index .. ".mp3"
local file = io.open(tempFile, "wb")
if file then
file:write(content)
file:close()
table.insert(tempFiles, tempFile)
processedChunks = processedChunks + 1
updateProgress("Completado fragmento " .. processedChunks .. " de " .. totalChunks)
if not failed and not shouldCancelGeneration then processChunk(index + 1) end
else
generationInProgress = false
updateExitButtonText()
failed = true
cleanupTempFiles(tempFiles)
local errorMsg = "Error al guardar fragmento " .. index
showProgressNotification("Error en Generación TTS", errorMsg .. " - Intente de nuevo", true)
callback(nil, errorMsg)
end
end)
end
}, 500)
end
end
processChunk(1)
end

function hideKeyboard(view)
pcall(function()
local imm = service.getSystemService(Context.INPUT_METHOD_SERVICE)
if imm and view then imm.hideSoftInputFromWindow(view.getWindowToken(), 0) end
end)
end

function showErrorInterface(errorMessage)
local errorLayout = {
LinearLayout,
orientation = "vertical",
layout_width = "fill",
layout_height = "fill",
padding = "20dp",
gravity = "center",
{
TextView,
text = "Error en la Generación",
layout_width = "wrap_content",
layout_height = "wrap_content",
textSize = "18sp",
gravity = "center",
layout_marginBottom = "20dp"
},
{
TextView,
text = errorMessage,
layout_width = "wrap_content",
layout_height = "wrap_content",
textSize = "14sp",
gravity = "center",
layout_marginBottom = "30dp"
},
{
Button,
id = "retryButton",
text = "Intentar generar un nuevo audio",
layout_width = "250dp",
layout_height = "wrap_content",
layout_marginBottom = "15dp"
},
{
Button,
id = "errorCloseButton",
text = "Cerrar",
layout_width = "150dp",
layout_height = "wrap_content"
}
}
local errorDlg = LuaDialog(service)
errorDlg.setTitle("Error de Generación")
errorDlg.setView(loadlayout(errorLayout))
retryButton.onClick = function()
vibrate()
errorDlg.dismiss()
currentErrorMessage = ""
showMainInterface()
end
errorCloseButton.onClick = function()
vibrate()
currentErrorMessage = ""
errorDlg.dismiss()
end
errorDlg.show()
end

function showMainInterface()
textInput.text = ""
fileNameInput.text = ""
resultText.text = ""
generateAudioButton.text = "Generar Audio"
toggleAudioControls(false)
updateExitButtonText()
dlg.show()
end

function showIndependentInterface()
local displayFileName = sanitizeFileName(savedFileName)
local independentLayout = {
LinearLayout,
orientation = "vertical",
layout_width = "fill",
layout_height = "fill",
padding = "20dp",
gravity = "center",
{
TextView,
text = "Audio Generado Exitosamente",
layout_width = "wrap_content",
layout_height = "wrap_content",
textSize = "18sp",
gravity = "center",
layout_marginBottom = "20dp"
},
{
TextView,
id = "audioInfoText",
text = "Archivo: " .. displayFileName .. ".mp3",
layout_width = "wrap_content",
layout_height = "wrap_content",
textSize = "14sp",
gravity = "center",
layout_marginBottom = "20dp"
},
{
Button,
id = "independentPlayButton",
text = "Reproducir",
layout_width = "200dp",
layout_height = "wrap_content",
layout_marginBottom = "10dp"
},
{
Button,
id = "independentDownloadButton",
text = "Descargar",
layout_width = "200dp",
layout_height = "wrap_content",
layout_marginBottom = "10dp"
},
{
Button,
id = "independentShareButton",
text = "Compartir",
layout_width = "200dp",
layout_height = "wrap_content",
layout_marginBottom = "20dp"
},
{
Button,
id = "independentCloseButton",
text = "Cerrar",
layout_width = "150dp",
layout_height = "wrap_content"
}
}
local independentDlg = LuaDialog(service)
independentDlg.setTitle("Audio Listo")
independentDlg.setView(loadlayout(independentLayout))
independentDlg.setOnKeyListener(DialogInterface.OnKeyListener {
onKey = function(dialog, keyCode, event)
if keyCode == KeyEvent.KEYCODE_BACK and event.getAction() == KeyEvent.ACTION_DOWN then
showProgressNotification("Aviso", "Solo puede cerrar con el botón Cerrar del plugin", true)
service.speak("Solo puede cerrar con el botón Cerrar del plugin")
return true
end
return false
end
})
local independentMediaPlayer = MediaPlayer()
independentPlayButton.onClick = function()
vibrate()
if savedAudioPath and File(savedAudioPath).exists() then
pcall(function()
if independentMediaPlayer.isPlaying() then
independentMediaPlayer.pause()
independentPlayButton.text = "Reproducir"
else
independentMediaPlayer.reset()
independentMediaPlayer.setDataSource(savedAudioPath)
independentMediaPlayer.prepareAsync()
independentMediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {
onPrepared = function()
independentMediaPlayer.start()
independentPlayButton.text = "Pausar"
end
})
independentMediaPlayer.setOnCompletionListener(MediaPlayer.OnCompletionListener {
onCompletion = function()
independentPlayButton.text = "Reproducir"
end
})
end
end)
else
service.speak("No hay audio disponible para reproducir")
end
end
independentDownloadButton.onClick = function()
vibrate()
if savedAudioData then
local fileName = sanitizeFileName(savedFileName)
local filePath = saveAudioToExternalStorage(savedAudioData, fileName)
if filePath then
service.speak("Audio guardado exitosamente en " .. filePath)
else
service.speak("Error al guardar el audio")
end
else
service.speak("No hay audio disponible para descargar")
end
end
independentShareButton.onClick = function()
vibrate()
local fileName = sanitizeFileName(savedFileName)
shareAudioFile(savedAudioData, fileName)
pcall(function()
if independentMediaPlayer.isPlaying() then independentMediaPlayer.stop() end
independentMediaPlayer.release()
end)
independentDlg.dismiss()
end
independentCloseButton.onClick = function()
vibrate()
pcall(function()
if independentMediaPlayer.isPlaying() then independentMediaPlayer.stop() end
independentMediaPlayer.release()
end)
independentDlg.dismiss()
end
independentDlg.show()
end

layout = {
ScrollView,
layout_width = "fill",
layout_height = "fill",
{
LinearLayout,
orientation = "vertical",
layout_width = "fill",
padding = "16dp",
layout_height = "wrap_content",
{
Spinner,
id = "serviceSpinner",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Spinner,
id = "categorySpinner",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Spinner,
id = "modelSpinner",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Spinner,
id = "styleSpinner",
layout_width = "fill",
layout_height = "wrap_content",
visibility = View.GONE
},
{
HorizontalScrollView,
id = "voiceScrollView",
layout_width = "fill",
layout_height = "wrap_content",
visibility = View.GONE,
{
LinearLayout,
id = "voiceButtonsLayout",
orientation = "horizontal",
layout_width = "wrap_content",
layout_height = "wrap_content",
padding = "8dp"
}
},
{
EditText,
id = "textInput",
hint = "Ingrese texto para convertir a audio",
layout_width = "fill",
layout_height = "wrap_content",
padding = "10dp",
gravity = "top",
lines = 10,
inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE
},
{
EditText,
id = "fileNameInput",
hint = "Ingrese nombre del archivo",
layout_width = "fill",
layout_height = "wrap_content",
padding = "10dp",
gravity = "top",
inputType = InputType.TYPE_CLASS_TEXT
},
{
Button,
id = "generateAudioButton",
text = "Generar Audio",
layout_width = "fill",
layout_height = "wrap_content"
},
{
TextView,
id = "resultText",
text = "",
layout_width = "fill",
layout_height = "wrap_content",
padding = "10dp"
},
{
Button,
id = "playPauseAudioButton",
text = "Reproducir",
layout_width = "wrap_content",
layout_height = "wrap_content",
visibility = View.GONE
},
{
Button,
id = "downloadButton",
text = "Descargar",
layout_width = "wrap_content",
layout_height = "wrap_content",
visibility = View.GONE
},
{
Button,
id = "shareButton",
text = "Compartir",
layout_width = "wrap_content",
layout_height = "wrap_content",
visibility = View.GONE
},
{
Button,
id = "multiVoiceButton",
text = "Múltiples Voces: OFF",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Button,
id = "telegramButton",
text = "Grupo de actualización de extensión",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Button,
id = "vibrationButton",
text = "Vibración: Activada",
layout_width = "fill",
layout_height = "wrap_content"
},
{
Button,
id = "exitButton",
text = "Salir",
layout_width = "fill",
layout_height = "wrap_content"
}
}
}

dlg = LuaDialog(service)
dlg.setTitle("Generador TTS de ElevenLabs")
dlg.setView(loadlayout(layout))
dlg.setOnKeyListener(DialogInterface.OnKeyListener {
onKey = function(dialog, keyCode, event)
if keyCode == KeyEvent.KEYCODE_BACK and event.getAction() == KeyEvent.ACTION_DOWN then
showProgressNotification("Aviso", "Solo puede salir con el botón Salir del plugin", true)
service.speak("Solo puede salir con el botón Salir del plugin")
return true
end
return false
end
})

local services = {"ElevenLabs", "TikTok", "Voces del sistema"}
local serviceAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(services))
serviceSpinner.setAdapter(serviceAdapter)
serviceSpinner.setSelection(0)

local categories = {"Alemán", "Disney", "Español", "Francés", "Inglés", "Indonesio", "Japonés", "Coreano", "Otros", "Portugués", "Vietnamita"}
local categoryAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(categories))
categorySpinner.setAdapter(categoryAdapter)
categorySpinner.setSelection(2)

local styles = {"Estable", "Equilibrado", "Creativo"}
local styleAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(styles))
styleSpinner.setAdapter(styleAdapter)
styleSpinner.setSelection(1)

local models = {
["Inglés"] = {
{ name = "Game On", code = "en_male_jomboy" },
{ name = "Jessie", code = "en_us_002" },
{ name = "Warm", code = "es_mx_002" },
{ name = "Wacky", code = "en_male_funny" },
{ name = "Scream", code = "en_us_ghostface" },
{ name = "Empathetic", code = "en_female_samc" },
{ name = "Serious", code = "en_male_cody" },
{ name = "Beauty Guru", code = "en_female_makeup" },
{ name = "Bestie", code = "en_female_richgirl" },
{ name = "Trickster", code = "en_male_grinch" },
{ name = "Joey", code = "en_us_006" },
{ name = "Story Teller", code = "en_male_narration" },
{ name = "Mr. GoodGuy", code = "en_male_deadpool" },
{ name = "Narrator", code = "en_uk_001" },
{ name = "Male English UK", code = "en_uk_003" },
{ name = "Metro", code = "en_au_001" },
{ name = "Alfred", code = "en_male_jarvis" },
{ name = "ashmagic", code = "en_male_ashmagic" },
{ name = "olantekkers", code = "en_male_olantekkers" },
{ name = "Lord Cringe", code = "en_male_ukneighbor" },
{ name = "Mr. Meticulous", code = "en_male_ukbutler" },
{ name = "Debutante", code = "en_female_shenna" },
{ name = "Varsity", code = "en_female_pansino" },
{ name = "Marty", code = "en_male_trevor" },
{ name = "Pop Lullaby", code = "en_female_f08_twinkle" },
{ name = "Classic Electric", code = "en_male_m03_classical" },
{ name = "Bae", code = "en_female_betty" },
{ name = "Cupid", code = "en_male_cupid" },
{ name = "Granny", code = "en_female_grandma" },
{ name = "Cozy", code = "en_male_m2_xhxs_m03_christmas" },
{ name = "Author", code = "en_male_santa_narration" },
{ name = "Caroler", code = "en_male_sing_deep_jingle" },
{ name = "Santa", code = "en_male_santa_effect" },
{ name = "NYE 2023", code = "en_female_ht_f08_newyear" },
{ name = "Magician", code = "en_male_wizard" },
{ name = "Opera", code = "en_female_ht_f08_halloween" },
{ name = "Euphoric", code = "en_female_ht_f08_glorious" },
{ name = "Hypetrain", code = "en_male_sing_funny_it_goes_up" },
{ name = "Melodrama", code = "en_female_ht_f08_wonderful_world" },
{ name = "Quirky Time", code = "en_male_m2_xhxs_m03_silly" },
{ name = "Peaceful", code = "en_female_emotional" },
{ name = "Toon Beat", code = "en_male_m03_sunshine_soon" },
{ name = "Open Mic", code = "en_female_f08_warmy_breeze" },
{ name = "Jingle", code = "en_male_m03_lobby" },
{ name = "Thanksgiving", code = "en_male_sing_funny_thanksgiving" },
{ name = "Cottagecore", code = "en_female_f08_salut_damour" },
{ name = "Professor", code = "en_us_007" },
{ name = "Scientist", code = "en_us_009" },
{ name = "Confidence", code = "en_us_010" },
{ name = "Smooth", code = "en_au_002" }
},
["Disney"] = {
{ name = "Ghost Face", code = "en_us_ghostface" },
{ name = "Chewbacca", code = "en_us_chewbacca" },
{ name = "C3PO", code = "en_us_c3po" },
{ name = "Stitch", code = "en_us_stitch" },
{ name = "Stormtrooper", code = "en_us_stormtrooper" },
{ name = "Rocket", code = "en_us_rocket" },
{ name = "Madame Leota", code = "en_female_madam_leota" },
{ name = "Ghost Host", code = "en_male_ghosthost" },
{ name = "Pirate", code = "en_male_pirate" }
},
["Francés"] = {
{ name = "Francés - Masculino 1", code = "fr_001" },
{ name = "Francés - Masculino 2", code = "fr_002" }
},
["Español"] = {
{ name = "Español (España) - Masculino", code = "es_002" },
{ name = "Español MX - Masculino", code = "es_mx_002" }
},
["Alemán"] = {
{ name = "Alemán - Femenino", code = "de_001" },
{ name = "Alemán - Masculino", code = "de_002" }
},
["Portugués"] = {
{ name = "Portugués BR - Femenino 1", code = "br_001" },
{ name = "Portugués BR - Femenino 2", code = "br_003" },
{ name = "Portugués BR - Femenino 3", code = "br_004" },
{ name = "Portugués BR - Masculino", code = "br_005" },
{ name = "Ivete Sangalo", code = "bp_female_ivete" },
{ name = "Ludmilla", code = "bp_female_ludmilla" },
{ name = "Lhays Macedo", code = "pt_female_lhays" },
{ name = "Laizza", code = "pt_female_laizza" },
{ name = "Galvão Bueno", code = "pt_male_bueno" }
},
["Coreano"] = {
{ name = "Coreano - Masculino 1", code = "kr_002" },
{ name = "Coreano - Femenino", code = "kr_003" },
{ name = "Coreano - Masculino 2", code = "kr_004" }
},
["Vietnamita"] = {
{ name = "Femenino", code = "BV074_streaming" },
{ name = "Masculino", code = "BV075_streaming" }
},
["Otros"] = {
{ name = "Alto", code = "en_female_f08_salut_damour" },
{ name = "Tenor", code = "en_male_m03_lobby" },
{ name = "Sunshine Soon", code = "en_male_m03_sunshine_soon" },
{ name = "Warmy Breeze", code = "en_female_f08_warmy_breeze" },
{ name = "Glorious", code = "en_female_ht_f08_glorious" },
{ name = "It Goes Up", code = "en_male_sing_funny_it_goes_up" },
{ name = "Chipmunk", code = "en_male_m2_xhxs_m03_silly" },
{ name = "Dramatic", code = "en_female_ht_f08_wonderful_world" }
},
["Indonesio"] = {
{ name = "Indonesio - Femenino", code = "id_001" }
},
["Japonés"] = {
{ name = "Japonés - Femenino 1", code = "jp_001" },
{ name = "Japonés - Femenino 2", code = "jp_003" },
{ name = "Japonés - Femenino 3", code = "jp_005" },
{ name = "Japonés - Masculino", code = "jp_006" },
{ name = "りーさ", code = "jp_female_fujicochan" }
}
}

local elevenLabsModels = {
{ name = "Adam", code = "pNInz6obpgDQGcFmaJgB" },
{ name = "Aria", code = "21m00Tcm4TlvDq8ikWAM" },
{ name = "Roger", code = "CwhRBWXzGAHq8TQ4Fs17" },
{ name = "Sarah", code = "EXAVITQu4vr4xnSDxMaL" },
{ name = "Laura", code = "Yko7PKHZNXotIFUBG7I9" },
{ name = "Charlie", code = "IKne3meq5aSn9XLyUdCD" },
{ name = "George", code = "JBFqnCBsd6RMkjVDRZzb" },
{ name = "Callum", code = "N2lVS1w4EtoT3dr4eOWO" },
{ name = "River", code = "LcfcDJNUP1GQjkzn1xUU" },
{ name = "Liam", code = "TX3LPaxmHKxFdv7VOQHJ" },
{ name = "Charlotte", code = "XB0fDUnXU5powFXDhCwa" },
{ name = "Alice", code = "ErXwobaYiN019PkySvjV" },
{ name = "Matilda", code = "XrExE9yKIg1WjnnlVkGX" },
{ name = "Will", code = "bVMeCyTHy58xNoL34h3p" },
{ name = "Jessica", code = "KZyXbCR19Q4tzr1zYhNH" },
{ name = "Eric", code = "VR6AewLTigWG4xSOukaG" },
{ name = "Chris", code = "iP95p4xoKVk53GoZ742B" },
{ name = "Brian", code = "nPczCjzI2devNBz1zQrb" },
{ name = "Daniel", code = "onwK4e9ZLuTAKqWW03F9" },
{ name = "Lily", code = "pFZP5JQG7iQjIQuC4Bku" },
{ name = "Bill", code = "zrHiDhphv9ZnVXBqCLjz" }
}

function createVoiceButtons()
voiceButtonsLayout.removeAllViews()
for _, voice in ipairs(elevenLabsVoices) do
local button = Button(service)
button.setText(voice.name)
button.setBackgroundColor(0xFF4CAF50)
button.setTextColor(0xFFFFFFFF)
button.setPadding(16, 8, 16, 8)
local params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT)
params.setMargins(8, 0, 8, 0)
button.setLayoutParams(params)
button.setOnClickListener(View.OnClickListener {
onClick = function(v)
vibrate()
insertVoiceTag(voice.name)
end
})
voiceButtonsLayout.addView(button)
end
end

local function updateModelSpinner(selectedService)
if selectedService == "TikTok" then
categorySpinner.visibility = View.VISIBLE
styleSpinner.visibility = View.GONE
local selectedCategory = categorySpinner.getSelectedItem()
local modelList = models[selectedCategory]
if modelList then
local modelNames = {}
for i, model in ipairs(modelList) do
table.insert(modelNames, model.name)
end
local modelAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(modelNames))
modelSpinner.setAdapter(modelAdapter)
modelSpinner.setSelection(0)
end
elseif selectedService == "ElevenLabs" then
categorySpinner.visibility = View.GONE
styleSpinner.visibility = View.VISIBLE
local modelNames = {}
for i, model in ipairs(elevenLabsModels) do
table.insert(modelNames, model.name)
end
local modelAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(modelNames))
modelSpinner.setAdapter(modelAdapter)
modelSpinner.setSelection(0)
else
categorySpinner.visibility = View.GONE
styleSpinner.visibility = View.GONE
local modelNames = {}
if systemVoicesLoaded and #systemVoices > 0 then
for i, voice in ipairs(systemVoices) do
table.insert(modelNames, voice.name)
end
else
table.insert(modelNames, "Inicializando TTS del sistema...")
if not systemVoicesLoaded then
handler.postDelayed(Runnable {
run = function()
updateModelSpinner("Voces del sistema")
end
}, 3000)
end
end
local modelAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(modelNames))
modelSpinner.setAdapter(modelAdapter)
modelSpinner.setSelection(0)
end
end

serviceSpinner.onItemSelected = function(parent, view, position, id)
vibrate()
local selectedService = parent.getItemAtPosition(position)
updateModelSpinner(selectedService)
updateDialogTitle()
end

categorySpinner.onItemSelected = function(parent, view, position, id)
vibrate()
local selectedCategory = parent.getItemAtPosition(position)
local modelList = models[selectedCategory]
if modelList then
local modelNames = {}
for i, model in ipairs(modelList) do
table.insert(modelNames, model.name)
end
local modelAdapter = ArrayAdapter(service, android.R.layout.simple_spinner_dropdown_item, String(modelNames))
modelSpinner.setAdapter(modelAdapter)
modelSpinner.setSelection(0)
end
end

modelSpinner.onItemSelected = function(parent, view, position, id)
vibrate()
end

styleSpinner.onItemSelected = function(parent, view, position, id)
vibrate()
end

serviceSpinner.setSelection(0)
updateModelSpinner("ElevenLabs")
updateDialogTitle()
initializeSystemTTS()
createVoiceButtons()

local mediaPlayer = MediaPlayer()
local currentAudioData = nil
local currentAudioPath = nil

function updateResultText(result)
resultText.text = result
pcall(function()
service.speak(result)
end)
end

function toggleAudioControls(show)
playPauseAudioButton.visibility = show and View.VISIBLE or View.GONE
downloadButton.visibility = show and View.VISIBLE or View.GONE
shareButton.visibility = show and View.VISIBLE or View.GONE
end

function updateVibrationButtonText()
vibrationButton.text = vibrationEnabled and "Vibración: Activada" or "Vibración: Desactivada"
end

loadVibrationPreference()
loadMultiVoicePreference()
updateVibrationButtonText()
updateMultiVoiceButtonText()
updateVoiceRowVisibility()

generateAudioButton.onClick = function()
vibrate()
if generationInProgress then
cancelGeneration()
return
end
hideKeyboard(textInput)
hideKeyboard(fileNameInput)
local text = textInput.text
if #text == 0 then
updateResultText("Por favor, ingrese texto.")
return
end
savedFileName = fileNameInput.text
local selectedService = serviceSpinner.getSelectedItem()
local selectedModel = modelSpinner.getSelectedItem()
local selectedStyle = styleSpinner.getSelectedItem()
local modelCode = nil
if selectedService == "TikTok" then
local selectedCategory = categorySpinner.getSelectedItem()
for i, model in ipairs(models[selectedCategory]) do
if model.name == selectedModel then
modelCode = model.code
break
end
end
elseif selectedService == "ElevenLabs" then
for i, model in ipairs(elevenLabsModels) do
if model.name == selectedModel then
modelCode = model.code
break
end
end
else
modelCode = selectedModel
end
if not modelCode or modelCode == "Inicializando TTS del sistema..." then
updateResultText("Espere a que se inicialice el TTS del sistema o seleccione un modelo válido.")
return
end
setupCancelReceiver()
generationInProgress = true
generateAudioButton.text = "Cancelar Generación"
updateResultText("Iniciando generación de audio...")
toggleAudioControls(false)
updateExitButtonText()
generateTextToAudio(text, modelCode, selectedStyle, function(audioData, error)
generateAudioButton.text = "Generar Audio"
pcall(function()
if cancelReceiver then
service.unregisterReceiver(cancelReceiver)
cancelReceiver = nil
end
end)
if error then
updateResultText("Error: " .. error)
toggleAudioControls(false)
currentErrorMessage = error
if shouldShowIndependentInterface then
shouldShowIndependentInterface = false
showErrorInterface(error)
end
else
currentAudioData = audioData
savedAudioData = audioData
updateResultText("¡Audio generado con éxito!")
toggleAudioControls(true)
currentAudioPath = service.getCacheDir().getPath() .. "/final_audio.mp3"
savedAudioPath = currentAudioPath
local file = io.open(currentAudioPath, "wb")
if file then
file:write(audioData)
file:close()
playPauseAudioButton.tag = currentAudioPath
else
updateResultText("Error al crear archivo temporal")
end
end
end)
end

playPauseAudioButton.onClick = function()
vibrate()
local audioFilePath = playPauseAudioButton.tag
if audioFilePath then
pcall(function()
if mediaPlayer.isPlaying() then
mediaPlayer.pause()
playPauseAudioButton.text = "Reproducir"
updateResultText("Pausado")
else
mediaPlayer.reset()
mediaPlayer.setDataSource(audioFilePath)
mediaPlayer.prepareAsync()
mediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {
onPrepared = function()
mediaPlayer.start()
playPauseAudioButton.text = "Pausar"
updateResultText("Reproduciendo")
end
})
mediaPlayer.setOnCompletionListener(MediaPlayer.OnCompletionListener {
onCompletion = function()
playPauseAudioButton.text = "Reproducir"
updateResultText("Reproducción finalizada")
end
})
end
end)
else
updateResultText("No hay audio disponible")
end
end

downloadButton.onClick = function()
vibrate()
local fileName = savedFileName
if #fileName == 0 then
updateResultText("Por favor, ingrese un nombre de archivo.")
return
end
if currentAudioData then
local cleanFileName = sanitizeFileName(fileName)
local filePath = saveAudioToExternalStorage(currentAudioData, cleanFileName)
if filePath then
updateResultText("Audio guardado en: " .. filePath)
else
updateResultText("Error al guardar el audio.")
end
else
updateResultText("No hay audio disponible")
end
end

shareButton.onClick = function()
vibrate()
local fileName = sanitizeFileName(savedFileName)
shareAudioFile(currentAudioData, fileName)
pcall(function()
if mediaPlayer.isPlaying() then mediaPlayer.stop() end
mediaPlayer.release()
end)
dlg.dismiss()
end

multiVoiceButton.onClick = function()
vibrate()
multiVoiceEnabled = not multiVoiceEnabled
saveMultiVoicePreference(multiVoiceEnabled)
updateMultiVoiceButtonText()
updateVoiceRowVisibility()
local status = multiVoiceEnabled and "activado" or "desactivado"
updateResultText("Modo múltiples voces " .. status)
end

telegramButton.onClick = function()
vibrate()
pcall(function()
local url = "https://chat.whatsapp.com/LZ4AYoKRuDkINnBjEICuCR"
local intent = Intent(Intent.ACTION_VIEW)
intent.setData(Uri.parse(url))
service.startActivity(intent)
dlg.dismiss()
end)
end

vibrationButton.onClick = function()
vibrate()
vibrationEnabled = not vibrationEnabled
saveVibrationPreference(vibrationEnabled)
updateVibrationButtonText()
local status = vibrationEnabled and "activada" or "desactivada"
updateResultText("Vibración " .. status)
end

exitButton.onClick = function()
vibrate()
if generationInProgress then
shouldShowIndependentInterface = true
dlg.dismiss()
else
pcall(function()
if mediaPlayer.isPlaying() then mediaPlayer.stop() end
mediaPlayer.release()
if systemTTS then systemTTS.shutdown() end
end)
dlg.dismiss()
end
end

vibrate()
updateExitButtonText()
dlg.show()